/*
 *	Target Dismount v0.02
 *	by Heikki Kernen
 */

const int SCRIPT_GAME_VERSION = int(0x000400);

UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;
int screenWidth, screenHeight;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

float xPos = 0, yPos = 0, zPos = 0;

uint id = 0;
uint curTime = 0;

bool finished = false;

PhysicsObject *limb;

int score = 0;
int totalScore;
int limbMultiplier = 1;

// Called once on level initialization.
void onInit()
{
	screenWidth = getScreenWidth();
	screenHeight = getScreenHeight();
	medFont = getFont("medium");
	smallFont = getFont("small");
	medFontHeight = medFont->getFontHeight();
	smallFontHeight = smallFont->getFontHeight();

	setClearColor(0.5,0.7,0.8);

	setGravity(0, 0, -2.0);

	head = findObject("Head");
	neck = findObject("Neck");
	trunk = findObject("Trunk");
	rthigh = findObject("RightThigh");
	lthigh = findObject("LeftThigh");
	rarm = findObject("RightArm");
	larm = findObject("LeftArm");
	rfore = findObject("RightForearm");
	lfore = findObject("LeftForearm");
	rcalf = findObject("RightCalf");
	lcalf = findObject("LeftCalf");
	rfoot = findObject("RightFoot");
	lfoot = findObject("LeftFoot");
}

void onUpdate(uint time)
{
	curTime = time;
	trunk->getPosition(xPos, yPos, zPos);
}


bool onCollision(PhysicsObject *o1, PhysicsObject *o2) {


	if (id == 0) {
		if (o1->getType() == PHYSOBJ_RAGDOLL_BODY) {
			id = o2->getId();
			limb = o1;
		}

		if (o2->getType() == PHYSOBJ_RAGDOLL_BODY) {
			id = o1->getId();
			limb = o2;
		}


		if (id >= 1 && id <= 5) score = 1000 * int(id);
		if (id >= 6 && id <= 9) score = 1000 * int(id) - 3000;
		if (id >= 10 && id <= 12) score = 1000 * int(id) - 5000;
		if (id >= 13 && id <= 14) score = 1000 * int(id) - 6000;
		if (id == 15) score = 10000;

		if (id != 0) {
			limbMultiplier = getLimbMultiplier();
			totalScore = score * limbMultiplier;
			finished = true;
		}
	}
	return true;
}

int getLimbMultiplier() {
	if (head == limb) return 2;
	else return 1;
}

void onRender()	{
	drawStuff();

	medFont->drawString(float(screenWidth)/2-1, 40-1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Target Dismount 0.02", 0,0,0,1);
	medFont->drawString(float(screenWidth)/2, 40, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Target Dismount 0.02", 1,1,1,0.5);

	if (finished) {

		medFont->drawString(float(screenWidth)/2 + 1,float(screenHeight)/2 - 30 + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landing successful!", 0,0,0,1);
		medFont->drawString(float(screenWidth)/2 + 1,float(screenHeight)/2 + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Base score: " + bstrFormat(score), 0,0,0,1);
		medFont->drawString(float(screenWidth)/2 + 1,float(screenHeight)/2 + 30 + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landed on " + limb->getName() + " (x" + bstrFormat(limbMultiplier) +")", 0,0,0,1);	
		medFont->drawString(float(screenWidth)/2 + 1,float(screenHeight)/2 + 60 + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Total score: " + bstrFormat(totalScore), 0,0,0,1);

		medFont->drawString(float(screenWidth)/2,float(screenHeight)/2 - 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landing successful!", 0,1,0,1);
		medFont->drawString(float(screenWidth)/2,float(screenHeight)/2, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Base score: " + bstrFormat(score), 1,1,1,1);
		medFont->drawString(float(screenWidth)/2,float(screenHeight)/2 + 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landed on " + limb->getName() + " (x" + bstrFormat(limbMultiplier) +")", 1,1,1,1);	
		medFont->drawString(float(screenWidth)/2,float(screenHeight)/2 + 60, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Total score: " + bstrFormat(totalScore), 0,0,1,1);

	}

}

void drawStuff() {

	// map offset
	int offset = 600;

	int x = int(xPos);
	int y = int(yPos);
	int z = int(zPos/2);

	// altitude
	int xOffset = offset+20+int(-xPos*2);
	int yOffset = int(120+yPos*2);
	drawPixel(xOffset, yOffset - z, 0,0,0,1);
	for (int y = yOffset - 100; y<yOffset; y++) {
		float color = 1 - float(y-20) / 120;
		drawPixel(xOffset-2, y, color,color,color,0.2);
		drawPixel(xOffset+2, y, color,color,color,0.2);
	}

	//start block
	drawPixel(offset+120,120, 1, 0, 1, 0.3);
	drawPixel(offset+120,121, 1, 0, 1, 0.3);
	drawPixel(offset+121,120, 1, 0, 1, 0.3);
	drawPixel(offset+121,121, 1, 0, 1, 0.3);

	// landing blocks
	for (int block = 0; block < 5; block++) {
		drawMap(offset - 70 * block, block * 5);
	}

	// X for the dummy
	drawX(xOffset,yOffset, 0, 0, 0, 0.5);

}

void drawMap (int offset, int start) {

	for (int i = start; i<20; i+=5) {
		for (int x = 0; x<40-i*2; x++) {
			drawPixel(offset+i+x,100+i, float(i)/15,1,0, 0.3);
			drawPixel(offset+i+x,100+40-i, float(i)/15,1,0, 0.3);
		}

		for (int y = 0; y<40-i*2; y++) {
			drawPixel(offset+i,100+i+y,float(i)/15,1,0, 0.3);
			drawPixel(offset+40-i,100+i+y,float(i)/15,1,0, 0.3);
		}
	}

	drawPixel(offset+20,120, 1, 0, 0, 0.3);
	drawPixel(offset+20,121, 1, 0, 0, 0.3);
	drawPixel(offset+21,120, 1, 0, 0, 0.3);
	drawPixel(offset+21,121, 1, 0, 0, 0.3);

}

void drawPixel(int x, int y, float r, float g, float b, float a) {
	smallFont->drawString(float(x), float(y), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, ".", r, g, b, a);
}

void drawX(int x, int y, float r, float g, float b, float a) {
	smallFont->drawString(float(x), float(y+4), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "X", r, g, b, a);
}
